/*
 * SPDX-FileCopyrightText: 2017~2017 CSSlayer <wengxt@gmail.com>
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 *
 */
#ifndef _IM_KEYSYMNAME_H_
#define _IM_KEYSYMNAME_H_

#include <fcitx-utils/key.h>
#include <string>

namespace fcitx {

std::string KeySymName(KeySym keyval);

} // namespace fcitx

#endif // _IM_KEYSYMNAME_H_
